/*
** 95io.c
** Released to the public domain on 07th of Sep. 1997.
** Set of routines to take advantage of the long filenames
** under windows95. Will not work in true DOS mode !!!
** Functions in general return 0 for success and non-zero
** for failure as DOS functions normally do.The return code
** generally is the DOS extended error code when !0.
** Getvol is the exception which i use to get LFN support info.
** All references taken from Ralph Brown's interrupt list.
** Comments, suggestions welcomed.
** All rights reserved on the comments, the rest is PD.
*/

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <io.h>
#include "95io.h"

struct W95FN lf_wreg;
W95VOL lf_info;

int LF_getosver(void)  /* check OS version */
{
 lf_wreg.regs.x.ax = 0x3306;
 LFINT86;
 return lf_wreg.regs.x.bx;
 }

/* also can be used to check if W95 LFN functions are allowed*/
int W95_getvol(int drive)
{
 char drv[4];
 sprintf(drv,"%c:\\\0",drive + 'A');
 lf_wreg.regs.x.ax = 0x71a0;
 lf_wreg.regs.x.dx = FP_OFF(drv);
 lf_wreg.sregs.ds = FP_SEG(drv);
 lf_wreg.regs.x.di = FP_OFF(lf_info.fatsys);
 lf_wreg.sregs.es = FP_SEG(lf_info.fatsys);
 lf_wreg.regs.x.cx = 32;
 LFINT86X;
 if(lf_wreg.regs.x.cflag)
 {
  if(lf_wreg.regs.x.ax == 0x7100)
	return LF_FALSE;
  }

 lf_info.lfinfo = lf_wreg.regs.x.bx;
 lf_info.filemax = lf_wreg.regs.x.cx;
 lf_info.pathmax = lf_wreg.regs.x.dx;

return LF_TRUE;
}

/* LFN  to Short (8.3) name conversion */
int LF_getdosname(char* dest,const char *path) /* dest to be 67 bytes */
{
 lf_wreg.regs.x.ax = 0x7160;
 lf_wreg.regs.x.cx = 0x01;
 lf_wreg.sregs.ds = FP_SEG(path);
 lf_wreg.regs.x.si = FP_OFF(path);
 lf_wreg.sregs.es = FP_SEG(dest);
 lf_wreg.regs.x.di = FP_OFF(dest);
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
}


/* LFN open file syntax as fopen(), alias will add a numeric
** descriptor for disambiguation under DOS short name.
** EG : myfilename.dat with alias 10 would become myfil~10.dat
*/
FILE* LF_fopen(const char *path, const char *fmode,unsigned alias)
{
 const char *p = fmode;
 char dosfn[67];

 lf_wreg.regs.x.bx = 0; /* assume "r" */

 if(*p == 'a')
  lf_wreg.regs.x.bx = 3;
  else
  while(*p)
  {
	switch (*p)
	{
	 case 'w' : lf_wreg.regs.x.bx |= 1; break;
	 case '+' : lf_wreg.regs.x.bx |= 2;
	 }
	p++;
	}

 switch(lf_wreg.regs.x.bx)     /* set file access action */
 {
  case 0 : lf_wreg.regs.x.dx = 1;  /* read only */
			  break;
  case 1 : lf_wreg.regs.x.dx = 18; /* write create/opentrunc */
			  break;
  case 2 : lf_wreg.regs.x.dx = 1;  /* r-w open/notrunc/fail noexist*/
			  break;
  case 3 : lf_wreg.regs.x.dx = 17; /* append open/create if noexist */
			  break;
  case 4 : lf_wreg.regs.x.dx = 1;  /* not implemented, but NT may make use of it */
  }

 if(alias)
 {
  lf_wreg.regs.x.bx |= 1024; /* use alias */
  lf_wreg.regs.x.di = alias;
  }

 lf_wreg.regs.x.bx |= 8192; /* ask for error codes to be retd */

 lf_wreg.regs.x.ax = 0x716c;
 lf_wreg.regs.x.cx = 32;    /* set archive bit */

 lf_wreg.sregs.ds = FP_SEG(path);
 lf_wreg.regs.x.si = FP_OFF(path);

 LFINT86X;

 if(lf_wreg.regs.x.cflag) /* if (ax == 0x7100) OS doesn't  */
 {                        /* support it, EG: bare msdos 7 */
  errno = lf_wreg.regs.x.ax;
  return NULL;
  }

 /* close the file so it can be re-opened with its DOS name */
 close(lf_wreg.regs.x.ax);

 //fclose((FILE*) lf_wreg.regs.x.ax); /* let stdio sort out the rest */

 if(LF_getdosname(dosfn,path))
 {
  return NULL;
  }

 return fopen(dosfn,fmode);  /* finally return a handle */
}
/*
** All the above crazyness is necessary at this stage, i found that the file
** handle returned by the LF_open is not stable under an older compiler
** like mine (BC4.02).. Don't know why, till then this is a safe bet.
*/



int LF_rename(const char *oldname,const char *newname)
{
 lf_wreg.regs.x.ax = 0x7156;
 lf_wreg.regs.x.dx = FP_OFF(oldname);
 lf_wreg.regs.x.di = FP_OFF(newname);
 lf_wreg.sregs.ds = FP_SEG(oldname);
 lf_wreg.sregs.es = FP_SEG(newname);
 lf_wreg.regs.x.cflag = 1;
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }


int LF_remove(const char *filepath)
{
 lf_wreg.regs.x.ax = 0x7141;
 lf_wreg.regs.x.dx = FP_OFF(filepath);
 lf_wreg.sregs.ds = FP_SEG(filepath);
 lf_wreg.regs.x.cflag = 1;
 lf_wreg.regs.x.si = 0;    /* no wild stuff as per stdio */
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }


int LF_mkdir(const char *path)
{
 lf_wreg.regs.x.ax = 0x7139;
 lf_wreg.regs.x.dx = FP_OFF(path);
 lf_wreg.sregs.ds = FP_SEG(path);
 lf_wreg.regs.x.cflag = 1;
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }


int LF_rmdir(const char *path)
{
 lf_wreg.regs.x.ax = 0x713a;
 lf_wreg.regs.x.dx = FP_OFF(path);
 lf_wreg.sregs.ds = FP_SEG(path);
 lf_wreg.regs.x.cflag = 1;
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }

int LF_chdir(const char *path)
{
 lf_wreg.regs.x.ax = 0x713b;
 lf_wreg.regs.x.dx = FP_OFF(path);
 lf_wreg.sregs.ds = FP_SEG(path);
 lf_wreg.regs.x.cflag = 1;
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }


int LF_getcurdir(int drive, char *directory)
{
 lf_wreg.regs.x.ax = 0x7147;
 lf_wreg.regs.h.dl = drive;
 lf_wreg.regs.x.si = FP_OFF(directory);
 lf_wreg.sregs.ds = FP_SEG(directory);
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }


int LF_getfileattrib(char *path, int *fattr)
{
 lf_wreg.regs.x.ax = 0x7143;
 lf_wreg.regs.x.dx = FP_OFF(path);
 lf_wreg.sregs.ds = FP_SEG(path);
 lf_wreg.regs.h.bl = 0;
 LFINT86X;
 if(lf_wreg.regs.x.cflag)
	return lf_wreg.regs.x.ax;

 *fattr = lf_wreg.regs.x.cx;
 return 0;
}


int LF_setfileattrib(char *path, int fattr)
{
 lf_wreg.regs.x.ax = 0x7143;
 lf_wreg.regs.x.dx = FP_OFF(path);
 lf_wreg.sregs.ds = FP_SEG(path);
 lf_wreg.regs.h.bl = 1;
 lf_wreg.regs.x.cx = fattr;
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }

/* Findfirst for long file names */
int LF_findfirst(struct _LFBLK *FindData, char *mask, int *handle)
{
 lf_wreg.regs.x.ax=0x714e;
 lf_wreg.regs.x.dx=FP_OFF(mask);
 lf_wreg.sregs.ds=FP_SEG(mask);
 lf_wreg.regs.x.di=FP_OFF(FindData);
 lf_wreg.sregs.es=FP_SEG(FindData);
 lf_wreg.regs.x.cx=0x00f7;
 lf_wreg.regs.x.si=1;
 lf_wreg.regs.x.cflag=1;
 LFINT86X;
 *handle=lf_wreg.regs.x.ax;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }

/* Findnext for long file names */
int LF_findnext(struct _LFBLK *FindData, int handle)
{
 lf_wreg.regs.x.ax=0x714f;
 lf_wreg.regs.x.bx=handle;
 lf_wreg.regs.x.di=FP_OFF(FindData);
 lf_wreg.sregs.es=FP_SEG(FindData);
 lf_wreg.regs.x.si=1;
 lf_wreg.regs.x.cflag=1;
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }

/* Terminate findfirst/findnext */
int LF_findclose(int handle)
{
 lf_wreg.regs.x.bx=handle;
 lf_wreg.regs.x.ax=0x71a1;
 lf_wreg.regs.x.cflag=1;
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
 }

/* Get the long file name, given the DOS name.
   Returns non-zero on error, as well as NULLing the longname.

   This simply uses the findfirst function. Function findclose
   must be called to terminate properly. */

int LF_getname(char *DOSname, char *longname)
{
struct _LFBLK FindData;
int handle;

 if(LF_findfirst(&FindData, DOSname, &handle))
 {
 	longname[0]='\0';
	return 1;
 }

 strcpy(longname, (char *)FindData.fname);
 LF_findclose(handle);
 return 0;
 }

int LF_setattrib(char *filename, int attribs)
{
 lf_wreg.regs.x.ax=0x7143;
 lf_wreg.regs.x.bx=1;
 lf_wreg.regs.x.cflag=1;
 lf_wreg.regs.x.cx=attribs;
 lf_wreg.regs.x.dx=FP_OFF(filename);
 lf_wreg.sregs.ds=FP_SEG(filename);
 LFINT86X;
 return lf_wreg.regs.x.cflag ? lf_wreg.regs.x.ax : 0;
}

int LF_getattrib(char *filename)
{
 lf_wreg.regs.x.ax=0x7143;
 lf_wreg.regs.x.bx=0;
 lf_wreg.regs.x.cflag=1;
 lf_wreg.regs.x.dx=FP_OFF(filename);
 lf_wreg.sregs.ds=FP_SEG(filename);
 LFINT86X;
 return lf_wreg.regs.x.cflag ? 0xffff : lf_wreg.regs.x.cx;
}
