/*
A file to alpha sort the user serial number list in Oscar 10.x
NOTE: Not tested with previous versions

The file to sort consists of two sections
The first contains the names of the files
The second contains the serial numbers etc.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <alloc.h>
#include <dir.h>

/* Include the file to save creating a makefile */
/* It's required to allow this DOS program to handle long file names */
#include "95io.c"

#define VERSION "1.1"

/* The base name used by Oscar */
#define NAME "YourSerials"

/* May as well be generous with these */
#define MAX 1000

int pointers[MAX];
long name_ptr[MAX];
long serial_ptr[MAX];

struct tbuf
{
	char n[513];
} huge *names;

/*
General purpose Shell/Metzner sort
In this case we use an index array for speed
*/
void sort_names(int total)
{
	int cnt1, cnt2, ptr1, ptr2;
	int m, k, j;
	int tmp;

	if(total<2)
		return;

	m = total;

  loop1:;
	m /= 2;

	if (!m)
		return;

	k = total - m;
	j = 1;

  loop2:;
	cnt1 = j;

  loop3:;
	cnt2 = cnt1 + m;
	ptr1 = cnt1 - 1;
	ptr2 = cnt2 - 1;

	/* This is the section which requires tailoring to the task */
	/* First the comparison */
	if (stricmp(names[pointers[ptr1]].n, names[pointers[ptr2]].n) > 0)
	{
		/* Then the swap */
		tmp=pointers[ptr1];
		pointers[ptr1]=pointers[ptr2];
		pointers[ptr2]=tmp;

		/* Back to common code */
		cnt1 -= m;

		if (cnt1 > 0)
			goto loop3;
	}

	j++;

	if (j > k)
		goto loop1;

	goto loop2;
}

int main(void)
{
int name=0, serial=0, cnt, total;
long pointer;
FILE *in, *out;
char buffer[513], *ptr;

	if((names=farmalloc((long)MAX*(long)sizeof(struct tbuf)))==NULL)
	{
		printf("\nMemory error (not enough?)\n");
		return 2;
	}

	LF_remove(NAME ".BAK");
	LF_rename(NAME ".DAT", NAME ".BAK");

	if((in=LF_fopen(NAME ".BAK", "rt", 0))==NULL)
	{
		printf("\nError opening source file (first pass)\n");
		return 1;
	}

	printf("Reading...\n");

	while(1)
	{
		/* Remember where we are in the file */
		pointer=ftell(in);

		if(fgets(buffer, sizeof(buffer), in)==NULL||!*buffer)
			break;

		/* Look for section headers */
		if(!strnicmp(buffer, "[name]", 6))
		{
			name++;
			cnt=0;
			continue;
		}

		if(!strnicmp(buffer, "[serial]", 8))
		{
			serial++;
			total=cnt;
			cnt=0;
			continue;
		}

		if((!name&&!serial)||*buffer==13||*buffer==10)
			continue;

		/* Copy the names into an array for sorting */
		if(name&&!serial)
		{
			/* Save the file pointer */
			pointers[cnt]=cnt;
			name_ptr[cnt]=pointer;

			if((ptr=strchr(buffer, '='))==NULL)
			{
				/* Something went wrong */
				printf("\nError parsing input line:\n%s\n", buffer);
				fclose(in);
				exit(1);
			}

			strcpy(names[cnt++].n, ++ptr);
			continue;
		}

		if(serial)
		{
			/* We only need the file pointer here */
			serial_ptr[cnt++]=pointer;
			continue;
		}
	}

	fclose(in);
	printf("Sorting %d items\n", total);
	sort_names(total);
	printf("Writing...\n");

	if((out=LF_fopen(NAME ".DAT", "wt", 0))==NULL)
	{
		printf("\nError opening destination file\n");
		return 1;
	}

	if((in=LF_fopen(NAME ".BAK", "rt", 0))==NULL)
	{
		printf("\nError opening source file (second pass)\n");
		fclose(out);
		return 1;
	}

	/* Copy everything up to, and including, the section header */
	while(fgets(buffer, sizeof(buffer), in)!=NULL)
	{
		fprintf(out, "%s", buffer);

		if(!strnicmp(buffer, "[name]", 6))
			break;
	}

	/* Write the names saved earlier */
	for(cnt=0; cnt<total; cnt++)
		fprintf(out, "%d=%s", cnt+1, names[pointers[cnt]].n);

	/* Now do the second section */
	fprintf(out, "\n[Serial]\n");

	for(cnt=0; cnt<total; cnt++)
	{
		/* Read the file using the sorted index pointers */
		fseek(in, serial_ptr[pointers[cnt]], 0);
		fgets(buffer, sizeof(buffer), in);

		/* Just to be sure */
		if((ptr=strchr(buffer, '='))==NULL)
		{
			printf("\nError parsing input line:\n%s\n", buffer);
			fclose(in);
			fclose(out);
			exit(1);
		}

		fprintf(out, "%d%s", cnt+1, ptr);
	}

	fclose(in);
	fclose(out);
	/* Delete or keep a backup of the original, your choice */
	/*LF_remove(NAME ".BAK");*/
	return 0;
}
